@@ -114,7 +114,7 @@ def nullspace(N, symbolic=True, atol=1e-13, rtol=0):
             nullspace_matrix = nullspace_matrix.row_insert(-1, nullspace_vectors[i].T)
 
         return to_fractions(
-            np.transpose(np.asarray(nullspace_matrix.rref()[0], dtype='object'))) if nullspace_matrix \
+            np.transpose(np.asarray(nullspace_matrix, dtype='object'))) if nullspace_matrix \
             else np.ndarray(shape=(N.shape[0], 0))
 
 
@@ -257,6 +257,12 @@ def redund(matrix, verbose=False):
 
     system('%s %s > %s' % (binary, matrix_path, matrix_nonredundant_path))
 
+    if not os.path.exists(matrix_nonredundant_path):
+        raise ValueError('An error occurred during removal of redundant vectors from an input matrix: '
+                         'redund did not write an output file after being presented input file "%s". \r\n\r\n'
+                         'Please check if your input matrix contains erroneous data, and let us know via https://github.com/SystemsBioinformatics/ecmtool/issues '
+                         'if you think the input matrix seems fine. It helps if you attach the matrix file mentioned above when creating an issue.')
+
     matrix_nored = np.ndarray(shape=(0, matrix.shape[1] + 1), dtype='object')
 
     with open(matrix_nonredundant_path) as file:
@@ -376,7 +382,7 @@ def print_ecms_direct(R, metabolite_ids):
         mp_print("")
 
 
-def normalize_columns(R, verbose=False):
+def normalize_columns_slower(R, verbose=False):  # This was the original function, but seems slower and further equivalent to new function below
     result = np.zeros(R.shape)
     number_rays = R.shape[1]
     for i in range(result.shape[1]):
@@ -387,7 +393,7 @@ def normalize_columns(R, verbose=False):
         largest_number = np.max(np.abs(R[:,i]))
         if largest_number > 1e100:  # If numbers are very large, converting to float might give issues, therefore we first divide by another int
             part_normalized_column = np.array(R[:, i] / largest_number, dtype='float')
-            result[:, i] = part_normalized_column / np.linalg.norm(part_normalized_column)
+            result[:, i] = part_normalized_column / np.linalg.norm(part_normalized_column, ord=1)
         else:
             norm_column = np.linalg.norm(np.array(R[:, i], dtype='float'), ord=1)
             if norm_column != 0:
@@ -395,6 +401,17 @@ def normalize_columns(R, verbose=False):
     return result
 
 
+def normalize_columns(R, verbose=False):
+    result = R
+    largest_number = max(np.max(R), -np.min(R))
+    if largest_number > 1e100:
+        result = result / largest_number # If numbers are very large, converting to float might give issues, therefore we first divide by another int
+    norms = np.linalg.norm(result, axis=0, ord=1)
+    norms[np.where(norms==0)[0]] = 1
+    result = np.array(np.divide(result, norms), dtype='float')
+    return result
+
+
 def find_remaining_rows(first_mat, second_mat, tol=1e-12, verbose=False):
     """Checks which rows (indices) of second_mat are still in first_mat"""
     remaining_inds = []
